@@ -1,4 +1,7 @@
-{-# LANGUAGE PatternGuards, FlexibleContexts, ScopedTypeVariables #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE FlexibleContexts    #-}
+{-# LANGUAGE PatternGuards       #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Eval
@@ -20,26 +23,29 @@ module Text.CSL.Eval
     , module Text.CSL.Eval.Output
     ) where
 
-import Control.Arrow
-import Control.Monad.State
-import Data.Monoid (Any(..))
-import Data.Char ( toLower, isDigit, isLetter )
-import Data.Maybe
-import Data.String ( fromString )
-import Text.Pandoc.Definition (Inline(Str, Link), nullAttr)
-import Text.Pandoc.Walk (walk)
-import Text.Pandoc.Shared (stringify)
-import qualified Data.Text as T
+import Prelude
+import           Control.Arrow
+import qualified Control.Exception      as E
+import           Control.Monad.State
+import           Data.Char              (isDigit, isLetter, toLower)
+import           Data.Maybe
+import           Data.Monoid            (Any (..))
+import           Data.String            (fromString)
+import qualified Data.Text              as T
+import           Text.Pandoc.Definition (Inline (Link, Str), nullAttr)
+import           Text.Pandoc.Shared     (stringify)
+import           Text.Pandoc.Walk       (walk)
 
-import Text.CSL.Eval.Common
-import Text.CSL.Eval.Output
-import Text.CSL.Eval.Date
-import Text.CSL.Eval.Names
-import Text.CSL.Output.Plain
-import Text.CSL.Reference
-import Text.CSL.Style hiding (Any)
-import Text.CSL.Util ( readNum, last', proc, proc', query, betterThan,
-                       safeRead, isRange )
+import           Text.CSL.Eval.Common
+import           Text.CSL.Eval.Date
+import           Text.CSL.Eval.Names
+import           Text.CSL.Eval.Output
+import           Text.CSL.Exception
+import           Text.CSL.Output.Plain
+import           Text.CSL.Reference
+import           Text.CSL.Style         hiding (Any)
+import           Text.CSL.Util          (betterThan, isRange, last', proc,
+                                         proc', query, readNum, safeRead)
 
 -- | Produce the output with a 'Layout', the 'EvalMode', a 'Bool'
 -- 'True' if the evaluation happens for disambiguation purposes, the
@@ -70,14 +76,21 @@ evalLayout (Layout _ _ es) em b l m o a mbr
                  (_,  Just "en") -> id
                  (_,  Nothing)   -> id
                  ("en", Just "") -> id
-                 _               -> proc' rmTitleCase
+                 _               -> proc' rmTitleCase'
 
 evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] ->
                [Sort] -> Abbreviations -> Maybe Reference -> [Sorting]
 evalSorting m l ms opts ss as mbr
     = map (format . sorting) ss
     where
-      render       = renderPlain . formatOutputList
+      render       = renderPlain . formatOutputList . proc removeDelimAndLabel
+      removeDelimAndLabel OLabel{} = ONull
+      removeDelimAndLabel ODel{}   = ONull
+      -- for sorting purposes, we need to distinguish between the space
+      -- inside a last name like ben Gurion, and the space between the
+      -- last name and the first.  OSpace is used for the latter.
+      removeDelimAndLabel OSpace{} = OStr "," emptyFormatting
+      removeDelimAndLabel x          = x
       format (s,e) = applaySort s . render $ uncurry eval e
       eval     o e = evalLayout (Layout emptyFormatting [] [e]) m False l ms o as mbr
       applaySort c s
@@ -113,9 +126,9 @@ evalElement el
                                        then getLocVar >>= formatRange fm . snd
                                        else formatNumber f fm s =<<
                                             getStringVar s
-    | Variable s f fm d     <- el = return . addDelim d =<< concatMapM (getVariable f fm) s
+    | Variable s f fm d     <- el = addDelim d <$> concatMapM (getVariable f fm) s
     | Group        fm d l   <- el = outputList fm d <$> tryGroup l
-    | Date     _ _ _  _ _ _ <- el = evalDate el
+    | Date{} <- el = evalDate el
     | Label    s f fm _     <- el = formatLabel f fm True s -- FIXME !!
     | Term     s f fm p     <- el = formatTerm  f fm p    s
     | Names    s n fm d sub <- el = modify (\st -> st { contNum = [] }) >>
@@ -136,7 +149,7 @@ evalElement el
     | Macro    s   fm       <- el = do
                         ms <- gets (macros . env)
                         case lookup s ms of
-                             Nothing  -> error $ "Macro " ++ show s ++ " not found!"
+                             Nothing  -> E.throw $ MacroNotFound (show s)
                              Just els -> do
                                res <- concat <$> mapM evalElement els
                                if null res
@@ -146,7 +159,7 @@ evalElement el
     where
       addSpaces strng = (if take 1 strng == " " then (OSpace:) else id) .
                         (if last' strng == " " then (++[OSpace]) else id)
-      substituteWith e = head <$> gets (names . env) >>= \(Names _ ns fm d _) -> do
+      substituteWith e = head <$> gets (names . env) >>= \(Names _ ns fm d _) ->
                            case e of
                              Names rs [Name NotSet fm'' [] [] []] fm' d' []
                                  -> let nfm = mergeFM fm'' $ mergeFM fm' fm in
@@ -172,8 +185,8 @@ evalElement el
                      nums <- mapM getStringVar numVars
                      let pluralizeTerm (Term s f fm _) = Term s f fm $
                             case numVars of
-                              ["number-of-volumes"] -> not $ any (== "1") nums
-                              ["number-of-pages"]   -> not $ any (== "1") nums
+                              ["number-of-volumes"] -> "1" `notElem` nums
+                              ["number-of-pages"]   -> "1" `notElem` nums
                               _ -> any isRange nums
                          pluralizeTerm x = x
                      if null res
@@ -192,7 +205,7 @@ evalElement el
           | Const {} <- e = True
           | otherwise     = False
 
-      ifEmpty p t e = p >>= \r -> if r == [] then t else return (e r)
+      ifEmpty p t e = p >>= \r -> if null r then t else return (e r)
 
       withNames e n f = modify (\s -> s { authSub = e ++ authSub s
                                         , env = (env s)
@@ -264,14 +277,16 @@ evalIfThen (IfThen c' m' el') ei e = whenElse (evalCond m' c') (return el') rest
                           in  getVar False chk "ref-type"
       chkNumeric      v = do val <- getStringVar v
                              as  <- gets (abbrevs . env)
-                             let val' = if getAbbreviation as v val == [] then val else getAbbreviation as v val
+                             let val' = if null (getAbbreviation as v val)
+                                           then val
+                                           else getAbbreviation as v val
                              return (isNumericString val')
-      chkDate         v = getDateVar v >>= return . not . null . filter circa
+      chkDate         v = any circa <$> getDateVar v
       chkPosition     s = if s == "near-note"
                           then gets (nearNote . cite . env)
-                          else gets (citePosition . cite . env) >>= return . compPosition s
-      chkDisambiguate s = gets disamb  >>= return . (==) (formatVariable s) . map toLower . show
-      chkLocator      v = getLocVar    >>= return . (==) v . fst
+                          else compPosition s <$> gets (citePosition . cite . env)
+      chkDisambiguate s = (==) (formatVariable s) . map toLower . show <$> gets disamb
+      chkLocator      v = (==) v . fst <$> getLocVar
       isIbid          s = not (s == "first" || s == "subsequent")
       compPosition a b
           | "first"             <- a = b == "first"
@@ -285,13 +300,13 @@ getFormattedValue o as f fm s val
     | Just v <- fromValue val :: Maybe Formatted =
        if v == mempty
           then []
-          else let ys = maybe (unFormatted v) (unFormatted . fromString)
+          else let ys = unFormatted . maybe v fromString
                         $ getAbbr (stringify $ unFormatted v)
                in  if null ys
                       then []
                       else [Output [OPan $ walk value' ys] fm]
-    | Just v <- fromValue val :: Maybe String    = (:[]) . flip OStr fm . maybe v id . getAbbr $ value v
-    | Just v <- fromValue val :: Maybe Literal   = (:[]) . flip OStr fm . maybe (unLiteral v) id . getAbbr $ value $ unLiteral v
+    | Just v <- fromValue val :: Maybe String    = (:[]) . flip OStr fm . Data.Maybe.fromMaybe v . getAbbr $ value v
+    | Just v <- fromValue val :: Maybe Literal   = (:[]) . flip OStr fm . Data.Maybe.fromMaybe (unLiteral v) . getAbbr $ value $ unLiteral v
     | Just v <- fromValue val :: Maybe Int       = output  fm (if v == 0 then [] else show v)
     | Just v <- fromValue val :: Maybe Int       = output  fm (if v == 0 then [] else show v)
     | Just v <- fromValue val :: Maybe CNum      = if v == 0 then [] else [OCitNum (unCNum v) fm]
@@ -306,8 +321,8 @@ getFormattedValue o as f fm s val
       value' x       = x
       getAbbr v = if f == Short
                   then case getAbbreviation as s v of
-                             []   -> Nothing
-                             y    -> Just y
+                             [] -> Nothing
+                             y  -> Just y
                   else Nothing
       nameOpts = ("name-as-sort-order","all") : o
       sortDate = [ DatePart "year"  "numeric-leading-zeros" "" emptyFormatting
@@ -318,10 +333,10 @@ formatTitle :: String -> Form -> Formatting -> State EvalState [Output]
 formatTitle s f fm
     | Short <- f
     , isTitleVar      s = try (getIt $ s ++ "-short") $ getIt s
-    | isTitleShortVar s = try (getIt s) $ return . (:[]) . flip OStr fm =<< getTitleShort s
+    | isTitleShortVar s = try (getIt s) $ (:[]) . flip OStr fm <$> getTitleShort s
     | otherwise         = getIt s
     where
-      try g h = g >>= \r -> if r == [] then h else return r
+      try g h = g >>= \r -> if null r then h else return r
       getIt x = do
         o <- gets (options . env)
         a <- gets (abbrevs . env)
@@ -331,24 +346,29 @@ formatNumber :: NumericForm -> Formatting -> String -> String -> State EvalState
 formatNumber f fm v n
     = gets (abbrevs . env) >>= \as ->
       if isNumericString (getAbbr as n)
-      then gets (terms . env) >>=
-           return . output fm . flip process (getAbbr as n)
+      then output fm . flip process (getAbbr as n) <$> gets (terms . env)
       else return . output fm . getAbbr as $ n
     where
-      getAbbr       as   = if getAbbreviation as v n == [] then id else getAbbreviation as v
+      getAbbr       as   = if null (getAbbreviation as v n)
+                              then id
+                              else getAbbreviation as v
       checkRange'   ts   = if v == "page" then checkRange ts else id
       process       ts   = checkRange' ts . printNumStr . map (renderNumber ts) .
                            breakNumericString . words
       renderNumber  ts x = if isTransNumber x then format ts x else x
 
       format tm = case f of
-                    Ordinal     -> ordinal     tm v
-                    LongOrdinal -> longOrdinal tm v
-                    Roman       -> if readNum n < 6000 then roman else id
-                    _           -> id
+                    Ordinal     -> maybe "" (ordinal     tm v) . safeRead
+                    LongOrdinal -> maybe "" (longOrdinal tm v) . safeRead
+                    Roman       -> maybe ""
+                                   (\x -> if x < 6000 then roman x else show x) .
+                                   safeRead
+                    _           -> maybe "" show . (safeRead :: String -> Maybe Int)
 
+      roman :: Int -> String
       roman     = foldr (++) [] . reverse . map (uncurry (!!)) . zip romanList .
-                  map (readNum . return) . take 4 . reverse
+                  map (readNum . return) . take 4 .
+                  reverse . show
       romanList = [[ "", "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix" ]
                   ,[ "", "x", "xx", "xxx", "xl", "l", "lx", "lxx", "lxxx", "xc" ]
                   ,[ "", "c", "cc", "ccc", "cd", "d", "dc", "dcc", "dccc", "cm" ]
@@ -364,7 +384,7 @@ checkRange ts (x:xs) = if x == '-' || x == '\x2013'
 
 printNumStr :: [String] -> String
 printNumStr []     = []
-printNumStr (x:[]) = x
+printNumStr [x] = x
 printNumStr (x:"-":y:xs) = x ++ "-"  ++ y ++ printNumStr xs
 printNumStr (x:",":y:xs) = x ++ ", " ++ y ++ printNumStr xs
 printNumStr (x:xs)
@@ -383,9 +403,9 @@ isTransNumber = all isDigit
 isSpecialChar = all (`elem` "&-,.\x2013")
 isNumber   cs = case [c | c <- cs
                         , not (isLetter c)
-                        , not (c `elem` "&-.,\x2013")] of
-                     []  -> False
-                     xs  -> all isDigit xs
+                        , c `notElem` "&-.,\x2013"] of
+                     [] -> False
+                     xs -> all isDigit xs
 
 breakNumericString :: [String] -> [String]
 breakNumericString [] = []
@@ -406,7 +426,7 @@ formatRange fm p = do
       pages = tupleRange . breakNumericString . words $ p
 
       tupleRange [] = []
-      tupleRange (x:cs:[]  )
+      tupleRange [x, cs]
         | cs `elem` ["-", "--", "\x2013"] = return (x,[])
       tupleRange (x:cs:y:xs)
         | cs `elem` ["-", "--", "\x2013"] = (x, y) : tupleRange xs
@@ -416,12 +436,12 @@ formatRange fm p = do
       joinRange (a,  b) = a ++ "-" ++ b
 
       process = checkRange ts . printNumStr . case opt of
-                 "expanded" -> map (joinRange . expandedRange)
-                 "chicago"  -> map (joinRange . chicagoRange )
-                 "minimal"  -> map (joinRange . minimalRange 1)
-                 "minimal-two"  -> map (joinRange . minimalRange 2)
-                 _          -> map joinRange
-  return [flip OLoc fm $ [OStr (process pages) emptyFormatting]]
+                 "expanded"    -> map (joinRange . expandedRange)
+                 "chicago"     -> map (joinRange . chicagoRange )
+                 "minimal"     -> map (joinRange . minimalRange 1)
+                 "minimal-two" -> map (joinRange . minimalRange 2)
+                 _             -> map joinRange
+  return [OLoc [OStr (process pages) emptyFormatting] fm]
 
 -- Abbreviated page ranges are expanded to their non-abbreviated form:
 -- 42–45, 321–328, 2787–2816
@@ -441,7 +461,7 @@ expandedRange (sa, sb)
 -- a minimum number of digits for the second number; thus, with
 -- minDigits = 2, we have 328-28.
 minimalRange :: Int -> (String, String) -> (String, String)
-minimalRange minDigits ((a:as), (b:bs))
+minimalRange minDigits (a:as, b:bs)
   | a == b
   , length as == length bs
   , length bs >= minDigits =
