@@ -1,4 +1,6 @@
-{-# LANGUAGE PatternGuards, FlexibleContexts #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE PatternGuards    #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Eval.Names
@@ -15,20 +17,21 @@
 
 module Text.CSL.Eval.Names where
 
-import Control.Monad.State
-import Data.Char  ( isLower, isUpper, isLetter )
-import Data.List  ( nub, intersperse )
-import Data.List.Split ( wordsBy )
-import Data.Maybe ( isJust )
+import Prelude
+import           Control.Monad.State
+import           Data.Char              (isLetter, isLower, isUpper)
+import           Data.List              (intersperse, nub)
+import           Data.List.Split        (wordsBy)
+import           Data.Maybe             (isJust)
 
-import Text.CSL.Eval.Common
-import Text.CSL.Eval.Output
-import Text.CSL.Util ( headInline, lastInline, readNum, (<^>), query, toRead,
-                       splitStrWhen, isRange )
-import Text.CSL.Style
-import Text.Pandoc.Definition
-import Text.Pandoc.Shared ( stringify )
-import qualified Text.Pandoc.Builder as B
+import           Text.CSL.Eval.Common
+import           Text.CSL.Eval.Output
+import           Text.CSL.Style
+import           Text.CSL.Util          (headInline, isRange, lastInline, query,
+                                         readNum, splitStrWhen, toRead, (<^>))
+import qualified Text.Pandoc.Builder    as B
+import           Text.Pandoc.Definition
+import           Text.Pandoc.Shared     (stringify)
 
 evalNames :: Bool -> [String] -> [Name] -> String -> State EvalState [Output]
 evalNames skipEdTrans ns nl d
@@ -55,7 +58,7 @@ evalNames skipEdTrans ns nl d
                   if null res
                     then     return []
                     else let role = if aus == ["author"] then "authorsub" else s
-                         in  return . return . OContrib k role res fb =<< gets etal
+                         in return . OContrib k role res fb <$> gets etal
         r'  <- evalNames skipEdTrans xs nl d
         num <- gets contNum
         return $ if r /= [] && r' /= []
@@ -64,7 +67,7 @@ evalNames skipEdTrans ns nl d
     | otherwise = return []
     where
       agents p s a = concatMapM (formatNames (hasEtAl nl) d p s a) nl
-      delim    ops = if d == [] then getOptionVal "names-delimiter" ops else d
+      delim    ops = if null d then getOptionVal "names-delimiter" ops else d
       resetEtal    = modify (\s -> s { etal = [] })
       count  num x = if hasCount nl && num /= [] -- FIXME!! le zero!!
                      then [OContrib [] [] [ONum (length num) emptyFormatting] [] []]
@@ -81,16 +84,19 @@ formatNames :: Bool -> Delimiter -> String -> String -> [Agent] -> Name -> State
 formatNames ea del p s as n
     | Name f _ ns _ _ <- n, Count <- f = do
         b <- isBib <$> gets mode
-        o <- gets (options . env) >>= return . mergeOptions ns
+        o <- mergeOptions ns <$> gets (options . env)
         modify $ \st -> st { contNum = nub $ (++) (take (snd $ isEtAl b o p as) as) $ contNum st }
         return []
 
     | Name f fm ns d np <- n = do
         b <- isBib <$> gets mode
-        o <- gets (options . env) >>= return . mergeOptions ns
+        o <- mergeOptions ns <$> gets (options . env)
         m <- gets mode
         let odel  = if del /= [] then del else getOptionVal "name-delimiter" o
-            del'  = if d   /= [] then d   else if odel == [] then ", " else odel
+            del'
+              | d   /= [] = d
+              | null odel = ", "
+              | otherwise = odel
             (_,i) = isEtAl b o p as
             form  = case f of
                       NotSet -> case getOptionVal "name-form" o of
@@ -100,8 +106,11 @@ formatNames ea del p s as n
             genName x = do etal' <- formatEtAl o ea "et-al" fm del' x
                            if null etal'
                               then do t <- getTerm False Long "and"
-                                      return $ delim t o del' $ format m o form fm np x
-                              else return $ (addDelim del' $ format m o form fm np x) ++ etal'
+                                      return $ delim t o del'
+                                             $ format m o form fm np x
+                              else return $
+                                    addDelim del' (format m o form fm np x)
+                                    ++ etal'
         setLastName o $ formatName m False f fm o np (last as)
         updateEtal =<< mapM genName [1 + i .. length as]
         genName i
@@ -136,8 +145,8 @@ formatNames ea del p s as n
       updateEtal x = modify $ \st ->
                      let x' = if length x == 1 then repeat $ head x else x
                      in st { etal = case etal st of
-                                         []  -> x
-                                         ys  -> zipWith (++) ys x'
+                                         [] -> x
+                                         ys -> zipWith (++) ys x'
                            }
       isWithLastName os
           | "true" <-       getOptionVal "et-al-use-last"  os
@@ -166,7 +175,7 @@ formatNames ea del p s as n
           | "text"   <- getOptionVal "and" os = " " ++ t ++ " "
           | "symbol" <- getOptionVal "and" os = " & "
           | otherwise                          = []
-      andStr' t d os = if andStr t os == [] then d else andStr t os
+      andStr' t d os = if null (andStr t os) then d else andStr t os
 
       formatEtAl o b t fm d i = do
         ln <- gets lastName
@@ -177,8 +186,8 @@ formatNames ea del p s as n
                     | otherwise            -> return []
            else et_al o b t fm d i
       et_al o b t fm d i
-          = when' (gets mode >>= return . not . isSorting) $
-            if (b || length as <= i)
+          = when' ( not . isSorting <$> gets mode) $
+            if b || length as <= i
             then return []
             else do x <- getTerm False Long t
                     when' (return $ x /= []) $
@@ -217,7 +226,7 @@ isEtAl b os p as
       etAlMin' x y = if b then etAlMin x else read $ getOptionVal' x y
       isOptionSet'  s1 s2 = if b
                             then isOptionSet s1 os
-                            else or $ (isOptionSet s1 os) : [(isOptionSet s2 os)]
+                            else or $ isOptionSet s1 os : [isOptionSet s2 os]
       getOptionVal' s1 s2 = if null (getOptionVal s1 os)
                             then getOptionVal s2 os
                             else getOptionVal s1 os
@@ -264,10 +273,9 @@ formatName m b f fm ops np n
                        Nothing
                          | isInit x  -> addIn x [Space] -- default
                        _ -> Space : x ++ [Space]
-      addIn x i = foldr hyphenate []
-                  $ map (\z -> Str (headInline z) : i)
-                  $ wordsBy (== Str "-")
-                  $ splitStrWhen (=='-') x
+      addIn x i = foldr (hyphenate . (\z -> Str (headInline z) : i)) []
+                     $ wordsBy (== Str "-")
+                     $ splitStrWhen (=='-') x
 
       sortSep g s = when_ g $ separator ++ addAffixes (g <+> s) "given" mempty
       separator   = if null (getOptionVal "sort-separator" ops)
@@ -281,13 +289,13 @@ formatName m b f fm ops np n
                         oPan' (unFormatted $ nameSuffix n) fm
       suffNoCom = when_ (nameSuffix n) $ OSpace : oPan' (unFormatted $ nameSuffix n) fm
 
-      onlyGiven = not (givenName n == mempty) && family == mempty
+      onlyGiven = givenName n /= mempty && family == mempty
       given     = if onlyGiven
                      then givenLong
                      else when_ (givenName  n) . Formatted . trimsp . fixsp . concatMap initial $ givenName n
       fixsp     (Space:Space:xs) = fixsp (Space:xs)
-      fixsp     (x:xs)       = x : fixsp xs
-      fixsp     []           = []
+      fixsp     (x:xs)           = x : fixsp xs
+      fixsp     []               = []
       trimsp = reverse . dropWhile (==Space) . reverse . dropWhile (==Space)
       givenLong = when_ (givenName  n) . mconcat . intersperse (Formatted [Space]) $ givenName n
       family    = familyName n
@@ -298,23 +306,23 @@ formatName m b f fm ops np n
                       c <= '\x5FF' ||
                       (c >= '\x1e00' && c <= '\x1fff')
       shortName = oPan' (unFormatted $ nondropping <+> family) (form "family")
-      longName g = if isSorting m
-                   then let firstPart = case getOptionVal "demote-non-dropping-particle" ops of
+
+      longName g
+        | isSorting m = let firstPart = case getOptionVal "demote-non-dropping-particle" ops of
                                            "never" -> nondropping <+> family  <+> dropping
                                            _       -> family  <+> dropping <+> nondropping
                         in oPan' (unFormatted firstPart) (form "family") <++> oPan' (unFormatted g) (form "given") <> suffCom
-                   else if (b && getOptionVal "name-as-sort-order" ops == "first") ||
-                           getOptionVal "name-as-sort-order" ops == "all"
-                        then let (fam,par) = case getOptionVal "demote-non-dropping-particle" ops of
-                                               "never"     -> (nondropping <+> family, dropping)
-                                               "sort-only" -> (nondropping <+> family, dropping)
-                                               _           -> (family, dropping <+> nondropping)
-                             in oPan' (unFormatted fam) (form "family") <> sortSep g par <> suffCom
-                          else let fam = addAffixes (dropping <+> nondropping <+> family) "family" suff
-                                   gvn = oPan' (unFormatted g) (form "given")
-                               in  if all isByzantine $ stringify $ unFormatted family
-                                   then gvn <++> fam
-                                   else fam <> gvn
+        | (b && getOptionVal "name-as-sort-order" ops == "first") ||
+         getOptionVal "name-as-sort-order" ops == "all" = let (fam,par) = case getOptionVal "demote-non-dropping-particle" ops of
+                                                                            "never"     -> (nondropping <+> family, dropping)
+                                                                            "sort-only" -> (nondropping <+> family, dropping)
+                                                                            _           -> (family, dropping <+> nondropping)
+                                                          in oPan' (unFormatted fam) (form "family") <> sortSep g par <> suffCom
+        | otherwise = let fam = addAffixes (dropping <+> nondropping <+> family) "family" suff
+                          gvn = oPan' (unFormatted g) (form "given")
+                      in  if all isByzantine $ stringify $ unFormatted family
+                          then gvn <++> fam
+                          else fam <> gvn
 
       disWithGiven = getOptionVal "disambiguate-add-givenname" ops == "true"
       initialize   = isJust (lookup "initialize-with" ops) && not onlyGiven
@@ -337,12 +345,19 @@ formatName m b f fm ops np n
 
 formatTerm :: Form -> Formatting -> Bool -> String -> State EvalState [Output]
 formatTerm f fm p s = do
-  t <- getTerm p f s
+  plural <- if s `elem` ["page", "volume", "issue"]
+               then do
+                 varset <- isVarSet s
+                 if varset
+                    then isRange <$> getStringVar s
+                    else return p
+               else return p
+  t <- getTerm plural f s
   return $ oStr' t fm
 
 formatLabel :: Form -> Formatting -> Bool -> String -> State EvalState [Output]
 formatLabel f fm p s
-    | "locator" <- s = when' (gets (citeLocator . cite . env) >>= return . (/=) []) $ do
+    | "locator" <- s = when' ( (/=) [] <$> gets (citeLocator . cite . env)) $ do
                        (l,v) <- getLocVar
                        form (\fm' -> return . flip OLoc emptyFormatting . output fm') id l (isRange v)
     | "page"    <- s = checkPlural
@@ -365,7 +380,7 @@ formatLabel f fm p s
                       v <- getStringVar s
                       format  s (isRange v)
       format      = form output id
-      form o g t b = return . o fm =<< g . period <$> getTerm (b && p) f t
+      form o g t b = o fm . g . period <$> getTerm (b && p) f t
       period      = if stripPeriods fm then filter (/= '.') else id
 
 (<+>) :: Formatted -> Formatted -> Formatted
@@ -373,9 +388,9 @@ Formatted [] <+> ss = ss
 s  <+> Formatted [] = s
 Formatted xs <+> Formatted ys =
   case lastInline xs of
-       "’"  -> Formatted (xs ++ ys)
-       "-"  -> Formatted (xs ++ ys)
-       _    -> Formatted (xs ++ [Space] ++ ys)
+       "’" -> Formatted (xs ++ ys)
+       "-" -> Formatted (xs ++ ys)
+       _   -> Formatted (xs ++ [Space] ++ ys)
 
 (<++>) :: [Output] -> [Output] -> [Output]
 [] <++> o  = o
