@@ -1,4 +1,6 @@
-{-# LANGUAGE OverloadedStrings, TupleSections #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE OverloadedStrings #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Parser
@@ -15,24 +17,26 @@
 module Text.CSL.Parser (readCSLFile, parseCSL, parseCSL',
                         parseLocale, localizeCSL)
 where
-import qualified Data.Text as T
-import qualified Data.ByteString.Lazy as L
-import qualified Data.Map as M
-import qualified Control.Exception as E
-import Control.Monad (when)
-import Data.Either (lefts, rights)
-import Data.Text (Text, unpack)
-import Text.CSL.Style hiding (parseNames)
-import Text.CSL.Util (toRead, findFile)
-import System.Directory (getAppUserDataDirectory)
-import qualified Text.XML as X
-import Data.Default
-import Text.Pandoc.Shared (safeRead)
-import Text.XML.Cursor
-import Data.Maybe (listToMaybe, fromMaybe)
-import Text.Pandoc.UTF8 (fromStringLazy)
-import Text.CSL.Compat.Pandoc (fetchItem)
-import Text.CSL.Data (getLocale)
+import Prelude
+import qualified Control.Exception      as E
+import           Control.Monad          (when)
+import qualified Data.ByteString.Lazy   as L
+import           Data.Default
+import           Data.Either            (lefts, rights)
+import qualified Data.Map               as M
+import           Data.Maybe             (fromMaybe, listToMaybe)
+import           Data.Text              (Text, unpack)
+import qualified Data.Text              as T
+import           System.Directory       (getAppUserDataDirectory)
+import           Text.CSL.Compat.Pandoc (fetchItem)
+import           Text.CSL.Data          (getLocale)
+import           Text.CSL.Exception
+import           Text.CSL.Style         hiding (parseNames)
+import           Text.CSL.Util          (findFile, toRead)
+import           Text.Pandoc.Shared     (safeRead)
+import           Text.Pandoc.UTF8       (fromStringLazy)
+import qualified Text.XML               as X
+import           Text.XML.Cursor
 
 -- | Parse a 'String' into a 'Style' (with default locale).
 parseCSL :: String -> Style
@@ -55,7 +59,7 @@ readCSLFile :: Maybe String -> FilePath -> IO Style
 readCSLFile mbLocale src = do
   csldir <- getAppUserDataDirectory "csl"
   mbSrc <- findFile [".", csldir] src
-  fetchRes <- fetchItem Nothing (fromMaybe src mbSrc)
+  fetchRes <- fetchItem (fromMaybe src mbSrc)
   f <- case fetchRes of
             Left err         -> E.throwIO err
             Right (rawbs, _) -> return $ L.fromChunks [rawbs]
@@ -64,15 +68,15 @@ readCSLFile mbLocale src = do
   let pickParentCur = get "link" >=> attributeIs (X.Name "rel" Nothing Nothing) "independent-parent"
   let parentCur = cur $/ get "info" &/ pickParentCur
   let parent' = concatMap (stringAttr "href") parentCur
-  when (parent' == src) $ do
-    error $ "Dependent CSL style " ++ src ++ " specifies itself as parent."
+  when (parent' == src) $
+    E.throwIO $ DependentStyleHasItselfAsParent src
   case parent' of
        ""  -> localizeCSL mbLocale $ parseCSLCursor cur
        y   -> do
            -- note, we insert locale from the dependent style:
            let mbLocale' = case stringAttr "default-locale" cur of
-                                  ""   -> mbLocale
-                                  x    -> Just x
+                                  "" -> mbLocale
+                                  x  -> Just x
            readCSLFile mbLocale' y
 
 parseCSL' :: L.ByteString -> Style
@@ -103,7 +107,7 @@ parseCSLCursor cur =
         defaultLocale = case cur $| laxAttribute "default-locale" of
                              (x:_) -> unpack x
                              []    -> "en-US"
-        author = case (cur $// get "info" &/ get "author") of
+        author = case cur $// get "info" &/ get "author" of
                       (x:_) -> CSAuthor (x $/ get "name" &/ string)
                                  (x $/ get "email" &/ string)
                                  (x $/ get "uri"   &/ string)
@@ -128,9 +132,7 @@ string = unpack . T.concat . content
 
 attrWithDefault :: Read a => Text -> a -> Cursor -> a
 attrWithDefault t d cur =
-  case safeRead (toRead $ stringAttr t cur) of
-       Just x   -> x
-       Nothing  -> d
+  fromMaybe d $ safeRead (toRead $ stringAttr t cur)
 
 stringAttr :: Text -> Cursor -> String
 stringAttr t cur =
@@ -177,16 +179,16 @@ parseElement cur =
   case node cur of
        X.NodeElement e ->
          case X.nameLocalName $ X.elementName e of
-              "term" -> parseTerm cur
-              "text" -> parseText cur
-              "choose" -> parseChoose cur
-              "group" -> parseGroup cur
-              "label" -> parseLabel cur
-              "number" -> parseNumber cur
+              "term"       -> parseTerm cur
+              "text"       -> parseText cur
+              "choose"     -> parseChoose cur
+              "group"      -> parseGroup cur
+              "label"      -> parseLabel cur
+              "number"     -> parseNumber cur
               "substitute" -> parseSubstitute cur
-              "names" -> parseNames cur
-              "date" -> parseDate cur
-              _ -> []
+              "names"      -> parseNames cur
+              "date"       -> parseDate cur
+              _            -> []
        _ -> []
 
 getFormatting :: Cursor -> Formatting
@@ -219,7 +221,7 @@ parseDate cur = [Date (words variable) form format delim parts partsAttr]
                            _         -> NoFormDate
         format     = getFormatting cur
         delim      = stringAttr "delimiter" cur
-        parts      = cur $/ get "date-part" &| (parseDatePart form)
+        parts      = cur $/ get "date-part" &| parseDatePart form
         partsAttr  = stringAttr "date-parts" cur
 
 parseDatePart :: DateForm -> Cursor -> DatePart
@@ -227,13 +229,13 @@ parseDatePart defaultForm cur =
   DatePart { dpName       = stringAttr "name" cur
            , dpForm       = case stringAttr "form" cur of
                                   ""  -> case defaultForm of
-                                              TextDate -> "long"
+                                              TextDate    -> "long"
                                               NumericDate -> "numeric"
-                                              _ -> "long"
+                                              _           -> "long"
                                   x    -> x
            , dpRangeDelim = case stringAttr "range-delimiter" cur of
-                                  ""  -> "-"
-                                  x   -> x
+                                  "" -> "-"
+                                  x  -> x
            , dpFormatting = getFormatting cur
            }
 
@@ -313,9 +315,7 @@ parseText cur =
               then [Macro macro formatting]
               else if not (null variable)
                       then [Variable (words variable) textForm formatting delim]
-                      else if not (null value)
-                           then [Const value formatting]
-                           else []
+                      else [Const value formatting | not (null value)]
 
 parseChoose :: Cursor -> [Element]
 parseChoose cur =
@@ -389,8 +389,8 @@ parseKey cur =
                        (stringAttr "names-use-last" cur)]
        x  -> [SortVariable x sorting]
   where sorting = case stringAttr "sort" cur of
-                       "descending"  -> Descending ""
-                       _             -> Ascending ""
+                       "descending" -> Descending ""
+                       _            -> Ascending ""
 
 parseBiblio :: Cursor -> Bibliography
 parseBiblio cur =
