@@ -1,11 +1,18 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, PatternGuards, OverloadedStrings,
-  DeriveDataTypeable, ExistentialQuantification, FlexibleInstances,
-  ScopedTypeVariables, GeneralizedNewtypeDeriving, IncoherentInstances,
-  DeriveGeneric, CPP #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE CPP                        #-}
+{-# LANGUAGE DeriveDataTypeable         #-}
+{-# LANGUAGE DeriveGeneric              #-}
+{-# LANGUAGE ExistentialQuantification  #-}
+{-# LANGUAGE FlexibleInstances          #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE IncoherentInstances        #-}
+{-# LANGUAGE OverloadedStrings          #-}
+{-# LANGUAGE PatternGuards              #-}
+{-# LANGUAGE ScopedTypeVariables        #-}
 #if MIN_VERSION_base(4,8,0)
 #define OVERLAPS {-# OVERLAPPING #-}
 #else
-{-# LANGUAGE OverlappingInstances #-}
+{-# LANGUAGE OverlappingInstances       #-}
 #define OVERLAPS
 #endif
 -----------------------------------------------------------------------------
@@ -29,11 +36,12 @@ module Text.CSL.Reference ( Literal(..)
                           , fromValue
                           , isValueSet
                           , Empty(..)
+                          , Season(..)
+                          , seasonToInt
                           , RefDate(..)
                           , handleLiteral
                           , toDatePart
                           , setCirca
-                          , mkRefDate
                           , RefType(..)
                           , CNum(..)
                           , CLabel(..)
@@ -44,36 +52,44 @@ module Text.CSL.Reference ( Literal(..)
                           , processCites
                           , setPageFirst
                           , setNearNote
+                          , parseEDTFDate
                           )
 where
 
-import Data.List  ( elemIndex, intercalate )
-import Data.List.Split ( splitWhen )
-import Data.Maybe ( fromMaybe             )
-import Data.Generics hiding (Generic)
-import GHC.Generics (Generic)
-import Data.Aeson hiding (Value)
-import qualified Data.Aeson as Aeson
-import Data.Aeson.Types (Parser)
-import qualified Data.Yaml.Builder as Y
-import Data.Yaml.Builder (ToYaml(..))
-import Control.Applicative ((<|>))
-import qualified Data.Text as T
-import Data.Text (Text)
-import qualified Data.Vector as V
-import Data.Char (toLower, isDigit)
-import Text.CSL.Style hiding (Number)
-import Text.CSL.Util (parseString, parseInt, parseBool, safeRead, readNum,
-                      inlinesToString, capitalize, camelize, uncamelize,
-                      (&=), mapping')
-import Text.Pandoc (Inline(Str))
-import Data.String
-import qualified Text.Parsec as P
-import qualified Text.Parsec.String as P
+import Prelude
+import           Control.Applicative ((<|>))
+import           Control.Monad       (guard, mplus, msum)
+import           Data.Aeson          hiding (Value)
+import qualified Data.Aeson          as Aeson
+import           Data.Aeson.Types    (Parser)
+import           Data.Char           (isDigit, toLower)
+import           Data.Either         (lefts, rights)
+import           Data.Generics       hiding (Generic)
 import qualified Data.HashMap.Strict as H
+import           Data.List           (elemIndex)
+import           Data.List.Split     (splitWhen)
+import           Data.Maybe          (fromMaybe, isNothing)
+import           Data.String
+import           Data.Text           (Text)
+import qualified Data.Text           as T
+import qualified Data.Vector         as V
+import           Data.Yaml.Builder   (ToYaml (..))
+import qualified Data.Yaml.Builder   as Y
+import           GHC.Generics        (Generic)
+import           Text.CSL.Style      hiding (Number)
+import           Text.CSL.Util       (camelize, capitalize, inlinesToString,
+                                      mapping', parseBool, parseInt, parseMaybeInt,
+                                      parseString, readNum, safeRead, trim,
+                                      uncamelize, AddYaml(..))
+import           Text.Pandoc         (Inline (Str))
+import qualified Text.Parsec         as P
+import qualified Text.Parsec.String  as P
 
 newtype Literal = Literal { unLiteral :: String }
-  deriving ( Show, Read, Eq, Data, Typeable, Monoid, Generic )
+  deriving ( Show, Read, Eq, Data, Typeable, Semigroup, Monoid, Generic )
+
+instance AddYaml Literal
+  where x &= (Literal y) = x &= (T.pack y)
 
 instance FromJSON Literal where
   parseJSON v             = Literal `fmap` parseString v
@@ -120,31 +136,85 @@ isValueSet val
 
 data Empty = Empty deriving ( Typeable, Data, Generic )
 
+data Season = Spring | Summer | Autumn | Winter | RawSeason String
+     deriving (Show, Read, Eq, Typeable, Data, Generic)
+
+instance ToYaml Season where
+  toYaml Spring = toYaml (1 :: Int)
+  toYaml Summer = toYaml (2 :: Int)
+  toYaml Autumn = toYaml (3 :: Int)
+  toYaml Winter = toYaml (4 :: Int)
+  toYaml (RawSeason s) = toYaml (T.pack s)
+
+seasonToInt :: Season -> Maybe Int
+seasonToInt Spring = Just 1
+seasonToInt Summer = Just 2
+seasonToInt Autumn = Just 3
+seasonToInt Winter = Just 4
+seasonToInt _      = Nothing
+
+intToSeason :: Int -> Maybe Season
+intToSeason 1 = Just Spring
+intToSeason 2 = Just Summer
+intToSeason 3 = Just Autumn
+intToSeason 4 = Just Winter
+intToSeason _  = Nothing
+
+pseudoMonthToSeason :: Int -> Maybe Season
+pseudoMonthToSeason n
+  | n >= 13 && n <= 16 = intToSeason (n - 12)
+  | n >= 21 && n <= 24 = intToSeason (n - 20)
+  | otherwise          = Nothing
+
+-- | Parse JSON value as Maybe Season.
+parseMaybeSeason :: Maybe Aeson.Value -> Parser (Maybe Season)
+parseMaybeSeason Nothing = return Nothing
+parseMaybeSeason (Just x) = do
+  mbn <- parseMaybeInt (Just x) <|> return Nothing
+  case mbn of
+       Just n -> case intToSeason n of
+                      Just s  -> return $ Just s
+                      Nothing -> fail $ "Could not read season: " ++ show n
+       Nothing -> do
+         s <- parseString x
+         if null s
+            then return Nothing
+            else return $ Just $ RawSeason s
+
 data RefDate =
-    RefDate { year   :: Literal
-            , month  :: Literal
-            , season :: Literal
-            , day    :: Literal
+    RefDate { year   :: Maybe Int
+            , month  :: Maybe Int
+            , season :: Maybe Season
+            , day    :: Maybe Int
             , other  :: Literal
             , circa  :: Bool
             } deriving ( Show, Read, Eq, Typeable, Data, Generic )
 
+instance AddYaml RefDate where
+  _ &= (RefDate Nothing Nothing Nothing Nothing o _) | o == mempty = id
+  x &= y = x &= y
+
 instance FromJSON RefDate where
-  parseJSON (Array v) =
+  parseJSON (Array v) = handlePseudoMonths <$>
      case fromJSON (Array v) of
-          Success [y]     -> RefDate <$> parseJSON y <*>
-                    pure "" <*> pure "" <*> pure "" <*> pure "" <*> pure False
-          Success [y,m]   -> RefDate <$> parseJSON y <*> parseJSON m <*>
-                    pure "" <*> pure "" <*> pure "" <*> pure False
-          Success [y,m,d] -> RefDate <$> parseJSON y <*> parseJSON m <*>
-                    pure "" <*> parseJSON d <*> pure "" <*> pure False
+          Success [y]     -> RefDate <$> parseMaybeInt y <*>
+                    pure Nothing <*> pure Nothing <*> pure Nothing <*>
+                    pure "" <*> pure False
+          Success [y,m]   -> RefDate <$> parseMaybeInt y <*> parseMaybeInt m <*>
+                    pure Nothing <*> pure Nothing <*> pure "" <*> pure False
+          Success [y,m,d] -> RefDate <$> parseMaybeInt y <*> parseMaybeInt m <*>
+                    pure Nothing <*> parseMaybeInt d <*> pure "" <*> pure False
           Error e         -> fail $ "Could not parse RefDate: " ++ e
           _               -> fail "Could not parse RefDate"
+     where handlePseudoMonths r =
+              case month r >>= pseudoMonthToSeason of
+                   Just s  -> r{ month = Nothing, season = Just s }
+                   Nothing -> r
   parseJSON (Object v) = RefDate <$>
-              v .:? "year" .!= "" <*>
-              v .:? "month" .!= "" <*>
-              v .:? "season" .!= "" <*>
-              v .:? "day" .!= "" <*>
+              (v .:? "year" >>= parseMaybeInt) <*>
+              (v .:? "month" >>= parseMaybeInt) <*>
+              (v .:? "season" >>= parseMaybeSeason) <*>
+              (v .:? "day" >>= parseMaybeInt) <*>
               v .:? "literal" .!= "" <*>
               ((v .: "circa" >>= parseBool) <|> pure False)
   parseJSON _ = fail "Could not parse RefDate"
@@ -161,67 +231,82 @@ instance ToJSON RefDate where
 -}
 
 instance ToYaml RefDate where
-  toYaml r = mapping' [ "year" &= year r
-                      , "month" &= month r
-                      , "season" &= season r
-                      , "day" &= day r
-                      , "literal" &= other r
-                      , "circa" &= T.pack (if circa r then "1" else "")
-                      ]
+  toYaml r = mapping'
+               [ "year" &= year r
+               , "month" &= month r
+               , "season" &= season r
+               , "day" &= day r
+               , "literal" &= other r
+               , "circa" &= circa r
+               ]
 
 instance OVERLAPS
          FromJSON [RefDate] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
   parseJSON (Object v) = do
+    raw' <- v .:? "raw"
     dateParts <- v .:? "date-parts"
     circa' <- (v .: "circa" >>= parseBool) <|> pure False
+    season' <- v .:? "season" >>= parseMaybeSeason
     case dateParts of
-         Just (Array xs) -> mapM (fmap (setCirca circa') . parseJSON)
-                            $ V.toList xs
-         _               -> handleLiteral <$> parseJSON (Object v)
-  parseJSON x          = parseJSON x >>= mkRefDate
+         Just (Array xs) | isNothing raw' && not (V.null xs)
+                          -> mapM (fmap (setCirca circa' .
+                                   maybe id setSeason season') . parseJSON)
+                             $ V.toList xs
+         _ -> case raw' of
+                  Nothing -> handleLiteral <$> parseJSON (Object v)
+                  Just r  -> return $ parseRawDate r
+  parseJSON x  = parseRawDate <$> parseJSON x
 
--- Zotero doesn't properly support date ranges, so a common
--- workaround is 2005_2007 or 2005_; support this as date range:
-handleLiteral :: RefDate -> [RefDate]
-handleLiteral d@(RefDate (Literal "") (Literal "") (Literal "")
-                         (Literal "") (Literal xs) b)
-  = case splitWhen (=='_') xs of
-         [x,y] | all isDigit x && all isDigit y &&
-                 not (null x) ->
-                 [RefDate (Literal x) mempty mempty mempty mempty b,
-                  RefDate (Literal y) mempty mempty mempty mempty b]
-         _ -> [d]
-handleLiteral d = [d]
+instance OVERLAPS
+         ToJSON [RefDate] where
+  toJSON = toJSONDate
+
+toJSONDate :: [RefDate] -> Aeson.Value
+toJSONDate [] = Array V.empty
+toJSONDate ds = object' $
+  [ "date-parts" .= dateparts | not (null dateparts) ] ++
+  ["circa" .= (1 :: Int) | any circa ds] ++
+  (case msum (map season ds) of
+        Just (RawSeason s) -> ["season" .= s]
+        _                  -> []) ++
+  (case mconcat (map other ds) of
+        Literal l | not (null l) -> ["literal" .= l]
+        _                        -> [])
+  where dateparts = filter (not . emptyDatePart) $ map toDatePart ds
+        emptyDatePart [] = True
+        emptyDatePart xs = all (== 0) xs
 
 toDatePart :: RefDate -> [Int]
 toDatePart refdate =
-    case (safeRead (unLiteral $ year refdate),
-          safeRead (unLiteral $ month refdate),
-          safeRead (unLiteral $ day refdate)) of
+    case (year refdate, month refdate
+           `mplus`
+          ((12+) <$> (season refdate >>= seasonToInt)),
+          day refdate) of
          (Just (y :: Int), Just (m :: Int), Just (d :: Int))
                                      -> [y, m, d]
          (Just y, Just m, Nothing)   -> [y, m]
          (Just y, Nothing, Nothing)  -> [y]
          _                           -> []
 
-instance OVERLAPS
-         ToJSON [RefDate] where
-  toJSON [] = Array V.empty
-  toJSON xs = object' $
-    case filter (not . null) (map toDatePart xs) of
-         []  -> ["literal" .= intercalate "; " (map (unLiteral . other) xs)]
-         dps -> (["date-parts" .= dps ] ++
-                 ["circa" .= (1 :: Int) | or (map circa xs)] ++
-                 ["season" .= s | s <- map season xs, s /= mempty])
+
+-- Zotero doesn't properly support date ranges, so a common
+-- workaround is 2005_2007 or 2005_; support this as date range:
+handleLiteral :: RefDate -> [RefDate]
+handleLiteral d@(RefDate Nothing Nothing Nothing Nothing (Literal xs) b)
+  = case splitWhen (=='_') xs of
+         [x,y] | all isDigit x && all isDigit y &&
+                 not (null x) ->
+                 [RefDate (safeRead x) Nothing Nothing Nothing mempty b,
+                  RefDate (safeRead y) Nothing Nothing Nothing mempty b]
+         _ -> [d]
+handleLiteral d = [d]
 
 setCirca :: Bool -> RefDate -> RefDate
 setCirca circa' rd = rd{ circa = circa' }
 
-mkRefDate :: Literal -> Parser [RefDate]
-mkRefDate z@(Literal xs)
-  | all isDigit xs = return [RefDate z mempty mempty mempty mempty False]
-  | otherwise      = return [RefDate mempty mempty mempty mempty z False]
+setSeason :: Season -> RefDate -> RefDate
+setSeason season' rd = rd{ season = Just season' }
 
 data RefType
     = NoType
@@ -266,13 +351,15 @@ instance Show RefType where
     show x = map toLower . uncamelize . showConstr . toConstr $ x
 
 instance FromJSON RefType where
+  -- found in one of the test cases:
+  parseJSON (String "film") = return MotionPicture
   parseJSON (String t) =
-    (safeRead (capitalize . camelize . T.unpack $ t)) <|>
+    safeRead (capitalize . camelize . T.unpack $ t) <|>
     fail ("'" ++ T.unpack t ++ "' is not a valid reference type")
   parseJSON v@(Array _) =
     fmap (capitalize . camelize . inlinesToString) (parseJSON v) >>= \t ->
-      (safeRead t <|>
-       fail ("'" ++ t ++ "' is not a valid reference type"))
+      safeRead t <|>
+       fail ("'" ++ t ++ "' is not a valid reference type")
   parseJSON _ = fail "Could not parse RefType"
 
 instance ToJSON RefType where
@@ -283,11 +370,11 @@ instance ToYaml RefType where
 
 -- For some reason, CSL is inconsistent about hyphens and underscores:
 handleSpecialCases :: String -> String
-handleSpecialCases "motion-picture" = "motion_picture"
-handleSpecialCases "musical-score" = "musical_score"
+handleSpecialCases "motion-picture"         = "motion_picture"
+handleSpecialCases "musical-score"          = "musical_score"
 handleSpecialCases "personal-communication" = "personal_communication"
-handleSpecialCases "legal-case" = "legal_case"
-handleSpecialCases x = x
+handleSpecialCases "legal-case"             = "legal_case"
+handleSpecialCases x                        = x
 
 newtype CNum = CNum { unCNum :: Int } deriving ( Show, Read, Eq, Num, Typeable, Data, Generic )
 
@@ -300,11 +387,7 @@ instance ToJSON CNum where
 instance ToYaml CNum where
   toYaml r = Y.string (T.pack $ show $ unCNum r)
 
-newtype CLabel = CLabel { unCLabel :: String } deriving ( Show, Read, Eq, Typeable, Data, Generic )
-
-instance Monoid CLabel where
-    mempty = CLabel mempty
-    mappend (CLabel a) (CLabel b) = CLabel (mappend a b)
+newtype CLabel = CLabel { unCLabel :: String } deriving ( Show, Read, Eq, Typeable, Data, Generic, Semigroup, Monoid )
 
 instance FromJSON CLabel where
   parseJSON x = CLabel `fmap` parseString x
@@ -318,81 +401,81 @@ instance ToYaml CLabel where
 -- | The 'Reference' record.
 data Reference =
     Reference
-    { refId               :: Literal
-    , refType             :: RefType
+    { refId                    :: Literal
+    , refType                  :: RefType
 
-    , author              :: [Agent]
-    , editor              :: [Agent]
-    , translator          :: [Agent]
-    , recipient           :: [Agent]
-    , interviewer         :: [Agent]
-    , composer            :: [Agent]
-    , director            :: [Agent]
-    , illustrator         :: [Agent]
-    , originalAuthor      :: [Agent]
-    , containerAuthor     :: [Agent]
-    , collectionEditor    :: [Agent]
-    , editorialDirector   :: [Agent]
-    , reviewedAuthor      :: [Agent]
+    , author                   :: [Agent]
+    , editor                   :: [Agent]
+    , translator               :: [Agent]
+    , recipient                :: [Agent]
+    , interviewer              :: [Agent]
+    , composer                 :: [Agent]
+    , director                 :: [Agent]
+    , illustrator              :: [Agent]
+    , originalAuthor           :: [Agent]
+    , containerAuthor          :: [Agent]
+    , collectionEditor         :: [Agent]
+    , editorialDirector        :: [Agent]
+    , reviewedAuthor           :: [Agent]
 
-    , issued              :: [RefDate]
-    , eventDate           :: [RefDate]
-    , accessed            :: [RefDate]
-    , container           :: [RefDate]
-    , originalDate        :: [RefDate]
-    , submitted           :: [RefDate]
+    , issued                   :: [RefDate]
+    , eventDate                :: [RefDate]
+    , accessed                 :: [RefDate]
+    , container                :: [RefDate]
+    , originalDate             :: [RefDate]
+    , submitted                :: [RefDate]
 
-    , title               :: Formatted
-    , titleShort          :: Formatted
-    , reviewedTitle       :: Formatted
-    , containerTitle      :: Formatted
-    , volumeTitle         :: Formatted
-    , collectionTitle     :: Formatted
-    , containerTitleShort :: Formatted
-    , collectionNumber    :: Formatted --Int
-    , originalTitle       :: Formatted
-    , publisher           :: Formatted
-    , originalPublisher   :: Formatted
-    , publisherPlace      :: Formatted
-    , originalPublisherPlace :: Formatted
-    , authority           :: Formatted
-    , jurisdiction        :: Formatted
-    , archive             :: Formatted
-    , archivePlace        :: Formatted
-    , archiveLocation     :: Formatted
-    , event               :: Formatted
-    , eventPlace          :: Formatted
-    , page                :: Formatted
-    , pageFirst           :: Formatted
-    , numberOfPages       :: Formatted
-    , version             :: Formatted
-    , volume              :: Formatted
-    , numberOfVolumes     :: Formatted --Int
-    , issue               :: Formatted
-    , chapterNumber       :: Formatted
-    , medium              :: Formatted
-    , status              :: Formatted
-    , edition             :: Formatted
-    , section             :: Formatted
-    , source              :: Formatted
-    , genre               :: Formatted
-    , note                :: Formatted
-    , annote              :: Formatted
-    , abstract            :: Formatted
-    , keyword             :: Formatted
-    , number              :: Formatted
-    , references          :: Formatted
-    , url                 :: Literal
-    , doi                 :: Literal
-    , isbn                :: Literal
-    , issn                :: Literal
-    , pmcid               :: Literal
-    , pmid                :: Literal
-    , callNumber          :: Literal
-    , dimensions          :: Literal
-    , scale               :: Literal
-    , categories          :: [Literal]
-    , language            :: Literal
+    , title                    :: Formatted
+    , titleShort               :: Formatted
+    , reviewedTitle            :: Formatted
+    , containerTitle           :: Formatted
+    , volumeTitle              :: Formatted
+    , collectionTitle          :: Formatted
+    , containerTitleShort      :: Formatted
+    , collectionNumber         :: Formatted --Int
+    , originalTitle            :: Formatted
+    , publisher                :: Formatted
+    , originalPublisher        :: Formatted
+    , publisherPlace           :: Formatted
+    , originalPublisherPlace   :: Formatted
+    , authority                :: Formatted
+    , jurisdiction             :: Formatted
+    , archive                  :: Formatted
+    , archivePlace             :: Formatted
+    , archiveLocation          :: Formatted
+    , event                    :: Formatted
+    , eventPlace               :: Formatted
+    , page                     :: Formatted
+    , pageFirst                :: Formatted
+    , numberOfPages            :: Formatted
+    , version                  :: Formatted
+    , volume                   :: Formatted
+    , numberOfVolumes          :: Formatted --Int
+    , issue                    :: Formatted
+    , chapterNumber            :: Formatted
+    , medium                   :: Formatted
+    , status                   :: Formatted
+    , edition                  :: Formatted
+    , section                  :: Formatted
+    , source                   :: Formatted
+    , genre                    :: Formatted
+    , note                     :: Formatted
+    , annote                   :: Formatted
+    , abstract                 :: Formatted
+    , keyword                  :: Formatted
+    , number                   :: Formatted
+    , references               :: Formatted
+    , url                      :: Literal
+    , doi                      :: Literal
+    , isbn                     :: Literal
+    , issn                     :: Literal
+    , pmcid                    :: Literal
+    , pmid                     :: Literal
+    , callNumber               :: Literal
+    , dimensions               :: Literal
+    , scale                    :: Literal
+    , categories               :: [Literal]
+    , language                 :: Literal
 
     , citationNumber           :: CNum
     , firstReferenceNoteNumber :: Int
@@ -480,8 +563,8 @@ instance FromJSON Reference where
        v .:? "citation-label" .!= mempty)
     where takeFirstNum (Formatted (Str xs : _)) =
             case takeWhile isDigit xs of
-                   []   -> mempty
-                   ds   -> Formatted [Str ds]
+                   [] -> mempty
+                   ds -> Formatted [Str ds]
           takeFirstNum x = x
           addPageFirst ref = if pageFirst ref == mempty && page ref /= mempty
                                 then ref{ pageFirst =
@@ -491,6 +574,8 @@ instance FromJSON Reference where
 
 -- Syntax for adding supplementary fields in note variable
 -- {:authority:Superior Court of California}{:section:A}{:original-date:1777}
+-- or
+-- Foo\nissued: 2016-03-20/2016-07-31\nbar
 -- see http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#supplementary-fields
 parseSuppFields :: Aeson.Object -> Parser Aeson.Object
 parseSuppFields o = do
@@ -501,20 +586,31 @@ parseSuppFields o = do
 
 noteFields :: P.Parser [(Text, Aeson.Value)]
 noteFields = do
-  fs <- P.many noteField
+  fs <- P.many (Right <$> (noteField <|> lineNoteField) <|> Left <$> regText)
   P.spaces
-  rest <- P.getInput
-  return (("note", Aeson.String (T.pack rest)) : fs)
+  let rest = T.unwords (lefts fs)
+  return (("note", Aeson.String rest) : rights fs)
 
 noteField :: P.Parser (Text, Aeson.Value)
 noteField = P.try $ do
-  P.spaces
-  P.char '{'
-  P.char ':'
+  _ <- P.char '{'
+  _ <- P.char ':'
   k <- P.manyTill (P.letter <|> P.char '-') (P.char ':')
+  _ <- P.skipMany (P.char ' ')
   v <- P.manyTill P.anyChar (P.char '}')
   return (T.pack k, Aeson.String (T.pack v))
 
+lineNoteField :: P.Parser (Text, Aeson.Value)
+lineNoteField = P.try $ do
+  _ <- P.char '\n'
+  k <- P.manyTill (P.letter <|> P.char '-') (P.char ':')
+  _ <- P.skipMany (P.char ' ')
+  v <- P.manyTill P.anyChar (P.char '\n' <|> '\n' <$ P.eof)
+  return (T.pack k, Aeson.String (T.pack v))
+
+regText :: P.Parser Text
+regText = (T.pack <$> P.many1 (P.noneOf "\n{")) <|> (T.singleton <$> P.anyChar)
+
 instance ToJSON Reference where
   toJSON ref = object' [
       "id" .= refId ref
@@ -820,7 +916,7 @@ setNearNote s cs
     = procGr [] cs
     where
       near_note   = let nn = fromMaybe [] . lookup "near-note-distance" . citOptions . citation $ s
-                    in  if nn == [] then 5 else readNum nn
+                    in  if null nn then 5 else readNum nn
       procGr _ [] = []
       procGr a (x:xs) = let (a',res) = procCs a x
                         in res : procGr a' xs
@@ -834,3 +930,127 @@ setNearNote s cs
                                   citeNoteNumber x /= "0" &&
                                   readNum (citeNoteNumber c) - readNum (citeNoteNumber x) <= near_note
                            _   -> False
+
+parseRawDate :: String -> [RefDate]
+parseRawDate o =
+  case P.parse rawDate "raw date" o of
+       Left _   -> [RefDate Nothing Nothing Nothing Nothing (Literal o) False]
+       Right ds -> ds
+
+rawDate :: P.Parser [RefDate]
+rawDate = rawDateISO <|> rawDateOld
+
+parseEDTFDate :: String -> [RefDate]
+parseEDTFDate o =
+  case handleRanges (trim o) of
+       [] -> []
+       o' -> case P.parse rawDateISO "date" o' of
+                Left _   -> []
+                Right ds -> ds
+    where handleRanges s =
+            case splitWhen (=='/') s of
+                 -- 199u EDTF format for a range
+                 [x] | 'u' `elem` x ->
+                      map (\c -> if c == 'u' then '0' else c) x ++ "/" ++
+                      map (\c -> if c == 'u' then '9' else c) x
+                 [x, "open"] -> x ++ "/"    -- EDTF
+                 [x, "unknown"] -> x ++ "/" -- EDTF
+                 _  -> s
+
+rawDateISO :: P.Parser [RefDate]
+rawDateISO = do
+  d1 <- isoDate
+  P.option [d1] (P.char '/' >>
+                  (\x -> [d1, x]) <$>
+                   (  isoDate <|> return emptydate )) <* P.eof
+   where emptydate = RefDate Nothing Nothing Nothing Nothing mempty False
+
+isoDate :: P.Parser RefDate
+isoDate = P.try $ do
+  extyear <- P.option False (True <$ P.char 'y')  -- EDTF year > 4 digits
+  -- needed for bibtex
+  y <- do
+    sign <- P.option "" (P.string "-")
+    rest <- P.count 4 P.digit
+    extended <- if extyear
+                   then P.many P.digit
+                   else return []
+    return $ case safeRead (sign ++ rest ++ extended) of
+                    Just x | x <= 0 -> Just (x - 1)  -- 0 = -1 AD
+                    x               -> x
+  m' <- P.option Nothing $ Just <$> P.try (P.char '-' >> P.many1 P.digit)
+  (m,s) <- case m' >>= safeRead of
+                   Just (n::Int)
+                          | n >= 1 && n <= 12  -> return (Just n, Nothing)
+                          | n >= 13 && n <= 16 -> return (Nothing, pseudoMonthToSeason n)
+                          | n >= 21 && n <= 24 -> return (Nothing, pseudoMonthToSeason n)
+                   Nothing | isNothing m' -> return (Nothing, Nothing)
+                   _ -> fail "Improper month"
+  d <- P.option Nothing $ safeRead <$> P.try (P.char '-' >> P.many1 P.digit)
+  guard $ case d of
+           Nothing -> True
+           Just (n::Int) | n >= 1 && n <= 31 -> True
+           _ -> False
+  P.optional $ do
+    _ <- P.char 'T'
+    _ <- P.many (P.digit <|> P.char ':')
+    P.optional $ (P.oneOf "+-" >> P.many1 (P.digit <|> P.char ':'))
+              <|> P.string "Z"
+  _ <- P.optional (P.char '?')
+  c <- P.option False (True <$ P.char '~')
+  return RefDate{ year = y, month = m,
+                  season = s, day = d,
+                  other = mempty, circa = c }
+
+rawDateOld :: P.Parser [RefDate]
+rawDateOld = do
+  let months   = ["jan","feb","mar","apr","may","jun","jul","aug",
+                  "sep","oct","nov","dec"]
+  let seasons  = ["spr","sum","fal","win"]
+  let pmonth = P.try $ do
+        xs <- P.many1 P.letter <|> P.many1 P.digit
+        if all isDigit xs
+           then case safeRead xs of
+                      Just (n::Int) | n >= 1 && n <= 12 -> return (Just n)
+                      _ -> fail "Improper month"
+           else case elemIndex (map toLower $ take 3 xs) months of
+                     Nothing -> fail "Improper month"
+                     Just n  -> return (Just (n+1))
+  let pseason = P.try $ do
+        xs <- P.many1 P.letter
+        case elemIndex (map toLower $ take 3 xs) seasons of
+             Just 0  -> return (Just Spring)
+             Just 1  -> return (Just Summer)
+             Just 2  -> return (Just Autumn)
+             Just 3  -> return (Just Winter)
+             _       -> fail "Improper season"
+  let pday = P.try $ do
+        xs <- P.many1 P.digit
+        case safeRead xs of
+             Just (n::Int) | n >= 1 && n <= 31 -> return (Just n)
+             _ -> fail "Improper day"
+  let pyear = safeRead <$> P.many1 P.digit
+  let sep = P.oneOf [' ','/',','] >> P.spaces
+  let rangesep = P.try $ P.spaces >> P.char '-' >> P.spaces
+  let refDate = RefDate Nothing Nothing Nothing Nothing mempty False
+  let date = P.choice $ map P.try [
+                 (do s <- pseason
+                     sep
+                     y <- pyear
+                     return refDate{ year = y, season = s })
+               , (do m <- pmonth
+                     sep
+                     d <- pday
+                     sep
+                     y <- pyear
+                     return refDate{ year = y, month = m, day = d })
+               , (do m <- pmonth
+                     sep
+                     y <- pyear
+                     return refDate{ year = y, month = m })
+               , (do y <- pyear
+                     return refDate{ year = y })
+               ]
+  d1 <- date
+  P.option [d1] ((\x -> [d1,x]) <$> (rangesep >> date))
+
