@@ -1,10 +1,17 @@
-{-# LANGUAGE OverloadedStrings, PatternGuards, DeriveDataTypeable,
-    ScopedTypeVariables, FlexibleInstances, DeriveGeneric,
-    GeneralizedNewtypeDeriving, CPP, MultiParamTypeClasses #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE CPP                        #-}
+{-# LANGUAGE DeriveDataTypeable         #-}
+{-# LANGUAGE DeriveGeneric              #-}
+{-# LANGUAGE FlexibleInstances          #-}
+
+{-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE OverloadedStrings          #-}
+{-# LANGUAGE PatternGuards              #-}
+{-# LANGUAGE ScopedTypeVariables        #-}
 #if MIN_VERSION_base(4,8,0)
 #define OVERLAPS {-# OVERLAPPING #-}
 #else
-{-# LANGUAGE OverlappingInstances #-}
+{-# LANGUAGE OverlappingInstances       #-}
 #define OVERLAPS
 #endif
 -----------------------------------------------------------------------------
@@ -64,6 +71,7 @@ module Text.CSL.Style ( readCSLString
                       , Formatting(..)
                       , emptyFormatting
                       , rmTitleCase
+                      , rmTitleCase'
                       , Quote(..)
                       , mergeFM
                       , CSInfo(..)
@@ -85,41 +93,44 @@ module Text.CSL.Style ( readCSLString
                       )
 where
 
-import Data.Aeson hiding (Number)
-import GHC.Generics (Generic)
-import Data.String
-import Control.Arrow hiding (left, right)
-import Control.Monad (liftM, mplus)
-import Control.Applicative ((<|>))
-import qualified Data.Aeson as Aeson
-import Data.Aeson.Types (Pair)
-import Data.List ( nubBy, isPrefixOf, isInfixOf, intersperse, intercalate )
-import Data.List.Split ( splitWhen, wordsBy )
-import Data.Generics ( Data, Typeable )
-import Data.Maybe ( listToMaybe )
-import qualified Data.Map as M
-import Data.Char (isPunctuation, isUpper, isLetter)
-import Text.CSL.Util (mb, parseBool, parseString, (.#?), (.#:), query,
-                      betterThan, trimr, tailInline, headInline,
-                      initInline, lastInline, splitStrWhen, mapping',
-                      (&=))
-import Data.Yaml.Builder(ToYaml(..))
-import qualified Data.Yaml.Builder as Y
-import Text.Pandoc.Definition hiding (Citation, Cite)
-import Text.Pandoc (bottomUp)
-import Text.CSL.Compat.Pandoc (readHtml, writeMarkdown)
-import qualified Text.Pandoc.Walk as Walk
-import qualified Text.Pandoc.Builder as B
-import qualified Data.Text as T
-import Text.Pandoc.XML (fromEntities)
+import Prelude
+import           Control.Applicative    ((<|>), (<$>))
+import           Control.Arrow          hiding (left, right)
+import           Control.Monad          (mplus)
+import           Data.Aeson             hiding (Number)
+import qualified Data.Aeson             as Aeson
+import           Data.Aeson.Types       (Pair)
+import           Data.Char              (isLetter, isPunctuation, isUpper, toLower)
+import qualified Data.Char              as Char
+import           Data.Generics          (Data, Typeable)
+import           Data.List              (intercalate, intersperse, isInfixOf,
+                                         isPrefixOf, nubBy)
+import           Data.List.Split        (splitWhen, wordsBy)
+import qualified Data.Map               as M
+import           Data.Maybe             (listToMaybe, isNothing)
+import           Data.String
+import qualified Data.Text              as T
+import           Data.Yaml.Builder      (ToYaml (..))
+import qualified Data.Yaml.Builder      as Y
+import           GHC.Generics           (Generic)
+import           Text.CSL.Compat.Pandoc (readHtml, writeMarkdown)
+import           Text.CSL.Util          (betterThan, headInline, initInline,
+                                         lastInline, mapping', mb, parseBool,
+                                         parseString, query, splitStrWhen,
+                                         tailInline, trimr, (.#:), (.#?),
+                                         AddYaml(..))
+import qualified Text.Pandoc.Builder    as B
+import           Text.Pandoc.Definition hiding (Citation, Cite)
+import qualified Text.Pandoc.Walk       as Walk
+import           Text.Pandoc.XML        (fromEntities)
 
 #ifdef UNICODE_COLLATION
-import qualified Data.Text     as T
-import qualified Data.Text.ICU as T
+import qualified Data.Text              as T
+import qualified Data.Text.ICU          as T
 #else
-import Data.RFC5051 (compareUnicode)
+import           Data.RFC5051           (compareUnicode)
 #endif
-import qualified Data.Vector as V
+import qualified Data.Vector            as V
 
 -- Note:  FromJSON reads HTML, ToJSON writes Markdown.
 -- This means that they aren't proper inverses of each other, which
@@ -130,9 +141,9 @@ import qualified Data.Vector as V
 readCSLString :: String -> [Inline]
 readCSLString s = Walk.walk handleSmallCapsSpans
                 $ case readHtml (adjustScTags s) of
-                        Pandoc _ [Plain ils]   -> ils
-                        Pandoc _ [Para  ils]   -> ils
-                        Pandoc _ x             -> Walk.query (:[]) x
+                        Pandoc _ [Plain ils] -> ils
+                        Pandoc _ [Para  ils] -> ils
+                        Pandoc _ x           -> Walk.query (:[]) x
   -- this is needed for versions of pandoc that don't turn
   -- a span with font-variant:small-caps into a SmallCaps element:
   where handleSmallCapsSpans (Span ("",[],[("style",sty)]) ils)
@@ -156,13 +167,13 @@ writeYAMLString :: [Inline] -> String
 writeYAMLString ils =
   trimr $ writeMarkdown
         $ Pandoc nullMeta
-          [Plain $ bottomUp (concatMap (adjustCSL False)) ils]
+          [Plain $ Walk.walk (concatMap (adjustCSL False)) ils]
 
 writeCSLString :: [Inline] -> String
 writeCSLString ils =
   trimr $ writeMarkdown
         $ Pandoc nullMeta
-          [Plain $ bottomUp (concatMap (adjustCSL True)) ils]
+          [Plain $ Walk.walk (concatMap (adjustCSL True)) ils]
 
 -- If the first param is True, we use special rich text conventions
 -- for CSL JSON, described here:
@@ -197,7 +208,7 @@ instance FromJSON Formatted where
   parseJSON v@(Array _) =
    Formatted <$> (parseJSON v
              <|> ((query (:[]) :: [Block] -> [Inline]) <$> parseJSON v))
-  parseJSON v           = fmap (Formatted . readCSLString) $ parseString v
+  parseJSON v           = (Formatted . readCSLString) Control.Applicative.<$> parseString v
 
 instance ToJSON Formatted where
   toJSON = toJSON . writeCSLString . unFormatted
@@ -208,14 +219,23 @@ instance ToYaml Formatted where
 instance IsString Formatted where
   fromString = Formatted . toStr
 
+instance AddYaml Formatted where
+  x &= (Formatted y) =
+           \acc -> if null y
+                      then acc
+                      else (x Y..= Formatted y) : acc
+
+instance Semigroup Formatted where
+  (<>) = appendWithPunct
+
 instance Monoid Formatted where
   mempty = Formatted []
-  mappend = appendWithPunct
+  mappend = (<>)
   mconcat = foldr mappend mempty
 
 instance Walk.Walkable Inline Formatted where
   walk f  = Formatted . Walk.walk f . unFormatted
-  walkM f = liftM Formatted . Walk.walkM f . unFormatted
+  walkM f = fmap Formatted . Walk.walkM f . unFormatted
   query f = Walk.query f . unFormatted
 
 instance Walk.Walkable Formatted Formatted where
@@ -240,7 +260,7 @@ toStr = intercalate [Str "\n"] .
 appendWithPunct :: Formatted -> Formatted -> Formatted
 appendWithPunct (Formatted left) (Formatted right) =
   Formatted $
-  case concat [lastleft, firstright] of
+  case lastleft ++ firstright of
        [' ',d] | d `elem` (",.:;" :: String) -> initInline left ++ right
        [c,d] | c `elem` (" ,.:;" :: String), d == c -> left ++ tailInline right
        [c,'.'] | c `elem` (",.!:;?" :: String) -> left ++ tailInline right
@@ -323,7 +343,7 @@ findTerm' s f g = findTerm'' s f (Just g)
 findTerm'' :: String -> Form -> Maybe Gender -> [CslTerm] -> Maybe CslTerm
 findTerm'' s f mbg ts
   = listToMaybe [ t | t <- ts, cslTerm t == s, termForm t == f,
-                         mbg == Nothing || mbg == Just (termGenderForm t) ]
+                         isNothing mbg || mbg == Just (termGenderForm t) ]
   `mplus`
   -- fallback: http://citationstyles.org/downloads/specification.html#terms
   case f of
@@ -426,7 +446,7 @@ data Match
 match :: Match -> [Bool] -> Bool
 match All  = and
 match Any  = or
-match None = and . map not
+match None = all not
 
 data DatePart
     = DatePart
@@ -466,16 +486,21 @@ instance Ord Sorting where
 compare' :: String -> String -> Ordering
 compare' x y
     = case (x, y) of
-        ('-':_,'-':_) -> comp (dropPunct y) (dropPunct x)
+        ('-':_,'-':_) -> comp (normalize y) (normalize x)
         ('-':_, _ )   -> LT
         (_  ,'-':_)   -> GT
-        _             -> comp (dropPunct x) (dropPunct y)
+        _             -> comp (normalize x) (normalize y)
       where
-        dropPunct = dropWhile isPunctuation
+        normalize = map (\c -> if c == ',' || c == '.' then ' ' else c) .
+                    filter (\c -> c == ',' ||
+                                  not (isPunctuation c || Char.isSpace c
+                                      -- ayn/hamza in transliterated arabic:
+                                       || c == 'ʾ' || c == 'ʿ'
+                                       ))
 #ifdef UNICODE_COLLATION
         comp a b = T.collate (T.collator T.Current) (T.pack a) (T.pack b)
 #else
-        comp a b = compareUnicode a b
+        comp a b = compareUnicode (map toLower a) (map toLower b)
 #endif
 
 data Form
@@ -590,6 +615,13 @@ instance Show Formatting where
 rmTitleCase :: Formatting -> Formatting
 rmTitleCase f = f{ textCase = if textCase f == "title" then "" else textCase f  }
 
+rmTitleCase' :: Output -> Output
+rmTitleCase' o =
+  case o of
+       OStr s f    -> OStr s (rmTitleCase f)
+       Output os f -> Output os (rmTitleCase f)
+       _           -> o
+
 data Quote
     = NativeQuote
     | ParsedQuote
@@ -696,7 +728,7 @@ instance FromJSON Cite where
 instance OVERLAPS
          FromJSON [[Cite]] where
   parseJSON (Array v) = mapM parseJSON $ V.toList v
-  parseJSON _ = return []
+  parseJSON _         = return []
 
 emptyCite :: Cite
 emptyCite  = Cite [] mempty mempty [] [] [] [] False False False 0
@@ -756,11 +788,11 @@ isPunctuationInQuote sty =
 
 object' :: [Pair] -> Aeson.Value
 object' = object . filter (not . isempty)
-  where isempty (_, Array v)  = V.null v
-        isempty (_, String t) = T.null t
+  where isempty (_, Array v)                                    = V.null v
+        isempty (_, String t)                                   = T.null t
         isempty ("first-reference-note-number", Aeson.Number n) = n == 0
-        isempty ("citation-number", Aeson.Number n) = n == 0
-        isempty (_, _)        = False
+        isempty ("citation-number", Aeson.Number n)             = n == 0
+        isempty (_, _)                                          = False
 
 data Agent
     = Agent { givenName       :: [Formatted]
@@ -777,16 +809,23 @@ data Agent
 emptyAgent :: Agent
 emptyAgent = Agent [] mempty mempty mempty mempty mempty False False
 
+-- CSL JSON uses quotes to protect capitalization
+removeQuoted :: Formatted -> Formatted
+removeQuoted (Formatted ils) = Formatted (go ils)
+  where go []                             = []
+        go (Quoted DoubleQuote ils' : xs) = ils' ++ go xs
+        go (x:xs)                         = x : go xs
+
 instance FromJSON Agent where
   parseJSON (Object v) = nameTransform <$> (Agent <$>
               (v .: "given" <|> ((map Formatted . wordsBy isSpace . unFormatted) <$> v .: "given") <|> pure []) <*>
               v .:?  "dropping-particle" .!= mempty <*>
               v .:? "non-dropping-particle" .!= mempty <*>
-              v .:? "family" .!= mempty <*>
+              (removeQuoted <$> (v .:? "family" .!= mempty)) <*>
               v .:? "suffix" .!= mempty <*>
               v .:? "literal" .!= mempty <*>
-              v .:? "comma-suffix" .!= False <*>
-              v .:? "parse-names" .!= False)
+              (v .:? "comma-suffix" >>= mb parseBool) .!= False <*>
+              (v .:? "parse-names" >>= mb parseBool) .!= False)
   parseJSON _ = fail "Could not parse Agent"
 
 instance ToYaml Agent where
@@ -851,18 +890,18 @@ droppingPartTransform ag
 
 startWithCapital' :: Inline -> Bool
 startWithCapital' (Str (c:_)) = isUpper c && isLetter c
-startWithCapital' _ = False
+startWithCapital' _           = False
 
 startWithCapital :: Formatted -> Bool
 startWithCapital (Formatted (x:_)) = startWithCapital' x
-startWithCapital _ = False
+startWithCapital _                 = False
 
 stripFinalComma :: Formatted -> (String, Formatted)
 stripFinalComma (Formatted ils) =
   case reverse $ splitStrWhen isPunctuation ils of
-       Str ",":xs -> (",", Formatted $ reverse xs)
+       Str ",":xs         -> (",", Formatted $ reverse xs)
        Str "!":Str ",":xs -> (",!", Formatted $ reverse xs)
-       _ -> ("", Formatted ils)
+       _                  -> ("", Formatted ils)
 
 suffixTransform :: Agent -> Agent
 suffixTransform ag
@@ -900,7 +939,8 @@ instance OVERLAPS
          FromJSON [Agent] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
   parseJSON (Object v) = (:[]) `fmap` parseJSON (Object v)
-  parseJSON _ = fail "Could not parse [Agent]"
+  parseJSON _          = fail "Could not parse [Agent]"
 
 -- instance ToJSON [Agent] where
 -- toJSON xs  = Array (V.fromList $ map toJSON xs)
+
